@@ -38,18 +38,18 @@ T = TypeVar('T')
 class RType:
     """Abstract base class for runtime types (erased, only concrete; no generics)."""
 
-    name = None  # type: str
+    name: str
     # If True, the type has a special unboxed representation. If False, the
     # type is represented as PyObject *. Even if True, the representation
     # may contain pointers.
     is_unboxed = False
     # This is the C undefined value for this type. It's used for initialization
     # if there's no value yet, and for function return value on error/exception.
-    c_undefined = None  # type: str
+    c_undefined: str
     # If unboxed: does the unboxed version use reference counting?
     is_refcounted = True
     # C type; use Emitter.ctype() to access
-    _ctype = None  # type: str
+    _ctype: str
 
     @abstractmethod
     def accept(self, visitor: 'RTypeVisitor[T]') -> T:
@@ -65,7 +65,7 @@ class RType:
         return '<%s>' % self.__class__.__name__
 
     def serialize(self) -> Union[JsonDict, str]:
-        raise NotImplementedError('Cannot serialize {} instance'.format(self.__class__.__name__))
+        raise NotImplementedError(f'Cannot serialize {self.__class__.__name__} instance')
 
 
 def deserialize_type(data: Union[JsonDict, str], ctx: 'DeserMaps') -> 'RType':
@@ -86,7 +86,7 @@ def deserialize_type(data: Union[JsonDict, str], ctx: 'DeserMaps') -> 'RType':
         elif data == "void":
             return RVoid()
         else:
-            assert False, "Can't find class {}".format(data)
+            assert False, f"Can't find class {data}"
     elif data['.class'] == 'RTuple':
         return RTuple.deserialize(data, ctx)
     elif data['.class'] == 'RUnion':
@@ -151,7 +151,7 @@ class RVoid(RType):
 
 
 # Singleton instance of RVoid
-void_rtype = RVoid()  # type: Final
+void_rtype: Final = RVoid()
 
 
 class RPrimitive(RType):
@@ -169,7 +169,7 @@ class RPrimitive(RType):
     """
 
     # Map from primitive names to primitive types and is used by deserialization
-    primitive_map = {}  # type: ClassVar[Dict[str, RPrimitive]]
+    primitive_map: ClassVar[Dict[str, "RPrimitive"]] = {}
 
     def __init__(self,
                  name: str,
@@ -232,12 +232,12 @@ class RPrimitive(RType):
 # little as possible, as generic ops are typically slow. Other types,
 # including other primitive types and RInstance, are usually much
 # faster.
-object_rprimitive = RPrimitive('builtins.object', is_unboxed=False,
-                               is_refcounted=True)  # type: Final
+object_rprimitive: Final = RPrimitive("builtins.object", is_unboxed=False, is_refcounted=True)
 
 # represents a low level pointer of an object
-object_pointer_rprimitive = RPrimitive('object_ptr', is_unboxed=False,
-                               is_refcounted=False, ctype='PyObject **')  # type: Final
+object_pointer_rprimitive: Final = RPrimitive(
+    "object_ptr", is_unboxed=False, is_refcounted=False, ctype="PyObject **"
+)
 
 # Arbitrary-precision integer (corresponds to Python 'int'). Small
 # enough values are stored unboxed, while large integers are
@@ -251,25 +251,31 @@ object_pointer_rprimitive = RPrimitive('object_ptr', is_unboxed=False,
 #
 # This cannot represent a subclass of int. An instance of a subclass
 # of int is coerced to the corresponding 'int' value.
-int_rprimitive = RPrimitive('builtins.int', is_unboxed=True, is_refcounted=True,
-                            ctype='CPyTagged')  # type: Final
+int_rprimitive: Final = RPrimitive(
+    "builtins.int", is_unboxed=True, is_refcounted=True, ctype="CPyTagged"
+)
 
 # An unboxed integer. The representation is the same as for unboxed
 # int_rprimitive (shifted left by one). These can be used when an
 # integer is known to be small enough to fit size_t (CPyTagged).
-short_int_rprimitive = RPrimitive('short_int', is_unboxed=True, is_refcounted=False,
-                                  ctype='CPyTagged')  # type: Final
+short_int_rprimitive: Final = RPrimitive(
+    "short_int", is_unboxed=True, is_refcounted=False, ctype="CPyTagged"
+)
 
 # Low level integer types (correspond to C integer types)
 
-int32_rprimitive = RPrimitive('int32', is_unboxed=True, is_refcounted=False,
-                              ctype='int32_t', size=4)  # type: Final
-int64_rprimitive = RPrimitive('int64', is_unboxed=True, is_refcounted=False,
-                              ctype='int64_t', size=8)  # type: Final
-uint32_rprimitive = RPrimitive('uint32', is_unboxed=True, is_refcounted=False,
-                               ctype='uint32_t', size=4)  # type: Final
-uint64_rprimitive = RPrimitive('uint64', is_unboxed=True, is_refcounted=False,
-                               ctype='uint64_t', size=8)  # type: Final
+int32_rprimitive: Final = RPrimitive(
+    "int32", is_unboxed=True, is_refcounted=False, ctype="int32_t", size=4
+)
+int64_rprimitive: Final = RPrimitive(
+    "int64", is_unboxed=True, is_refcounted=False, ctype="int64_t", size=8
+)
+uint32_rprimitive: Final = RPrimitive(
+    "uint32", is_unboxed=True, is_refcounted=False, ctype="uint32_t", size=4
+)
+uint64_rprimitive: Final = RPrimitive(
+    "uint64", is_unboxed=True, is_refcounted=False, ctype="uint64_t", size=8
+)
 
 # The C 'int' type
 c_int_rprimitive = int32_rprimitive
@@ -277,54 +283,60 @@ c_int_rprimitive = int32_rprimitive
 if IS_32_BIT_PLATFORM:
     c_size_t_rprimitive = uint32_rprimitive
     c_pyssize_t_rprimitive = RPrimitive('native_int', is_unboxed=True, is_refcounted=False,
-                              ctype='int32_t', size=4)
+                                        ctype='int32_t', size=4)
 else:
     c_size_t_rprimitive = uint64_rprimitive
     c_pyssize_t_rprimitive = RPrimitive('native_int', is_unboxed=True, is_refcounted=False,
-                              ctype='int64_t', size=8)
+                                        ctype='int64_t', size=8)
 
 # Low level pointer, represented as integer in C backends
-pointer_rprimitive = RPrimitive('ptr', is_unboxed=True, is_refcounted=False,
-                                ctype='CPyPtr')  # type: Final
+pointer_rprimitive: Final = RPrimitive("ptr", is_unboxed=True, is_refcounted=False, ctype="CPyPtr")
 
 # Floats are represent as 'float' PyObject * values. (In the future
 # we'll likely switch to a more efficient, unboxed representation.)
-float_rprimitive = RPrimitive('builtins.float', is_unboxed=False,
-                              is_refcounted=True)  # type: Final
+float_rprimitive: Final = RPrimitive("builtins.float", is_unboxed=False, is_refcounted=True)
 
 # An unboxed Python bool value. This actually has three possible values
 # (0 -> False, 1 -> True, 2 -> error). If you only need True/False, use
 # bit_rprimitive instead.
-bool_rprimitive = RPrimitive('builtins.bool', is_unboxed=True, is_refcounted=False,
-                             ctype='char', size=1)  # type: Final
+bool_rprimitive: Final = RPrimitive(
+    "builtins.bool", is_unboxed=True, is_refcounted=False, ctype="char", size=1
+)
 
 # A low-level boolean value with two possible values: 0 and 1. Any
 # other value results in undefined behavior. Undefined or error values
 # are not supported.
-bit_rprimitive = RPrimitive('bit', is_unboxed=True, is_refcounted=False,
-                            ctype='char', size=1)  # type: Final
+bit_rprimitive: Final = RPrimitive(
+    "bit", is_unboxed=True, is_refcounted=False, ctype="char", size=1
+)
 
 # The 'None' value. The possible values are 0 -> None and 2 -> error.
-none_rprimitive = RPrimitive('builtins.None', is_unboxed=True, is_refcounted=False,
-                             ctype='char', size=1)  # type: Final
+none_rprimitive: Final = RPrimitive(
+    "builtins.None", is_unboxed=True, is_refcounted=False, ctype="char", size=1
+)
 
 # Python list object (or an instance of a subclass of list).
-list_rprimitive = RPrimitive('builtins.list', is_unboxed=False, is_refcounted=True)  # type: Final
+list_rprimitive: Final = RPrimitive("builtins.list", is_unboxed=False, is_refcounted=True)
 
 # Python dict object (or an instance of a subclass of dict).
-dict_rprimitive = RPrimitive('builtins.dict', is_unboxed=False, is_refcounted=True)  # type: Final
+dict_rprimitive: Final = RPrimitive("builtins.dict", is_unboxed=False, is_refcounted=True)
 
 # Python set object (or an instance of a subclass of set).
-set_rprimitive = RPrimitive('builtins.set', is_unboxed=False, is_refcounted=True)  # type: Final
+set_rprimitive: Final = RPrimitive("builtins.set", is_unboxed=False, is_refcounted=True)
 
 # Python str object. At the C layer, str is referred to as unicode
 # (PyUnicode).
-str_rprimitive = RPrimitive('builtins.str', is_unboxed=False, is_refcounted=True)  # type: Final
+str_rprimitive: Final = RPrimitive("builtins.str", is_unboxed=False, is_refcounted=True)
+
+# Python bytes object.
+bytes_rprimitive: Final = RPrimitive('builtins.bytes', is_unboxed=False, is_refcounted=True)
 
 # Tuple of an arbitrary length (corresponds to Tuple[t, ...], with
 # explicit '...').
-tuple_rprimitive = RPrimitive('builtins.tuple', is_unboxed=False,
-                              is_refcounted=True)  # type: Final
+tuple_rprimitive: Final = RPrimitive("builtins.tuple", is_unboxed=False, is_refcounted=True)
+
+# Python range object.
+range_rprimitive: Final = RPrimitive("builtins.range", is_unboxed=False, is_refcounted=True)
 
 
 def is_tagged(rtype: RType) -> bool:
@@ -401,10 +413,18 @@ def is_str_rprimitive(rtype: RType) -> bool:
     return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.str'
 
 
+def is_bytes_rprimitive(rtype: RType) -> bool:
+    return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.bytes'
+
+
 def is_tuple_rprimitive(rtype: RType) -> bool:
     return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.tuple'
 
 
+def is_range_rprimitive(rtype: RType) -> bool:
+    return isinstance(rtype, RPrimitive) and rtype.name == 'builtins.range'
+
+
 def is_sequence_rprimitive(rtype: RType) -> bool:
     return isinstance(rtype, RPrimitive) and (
         is_list_rprimitive(rtype) or is_tuple_rprimitive(rtype) or is_str_rprimitive(rtype)
@@ -425,7 +445,7 @@ class TupleNameVisitor(RTypeVisitor[str]):
             return 'I'
         elif t._ctype == 'char':
             return 'C'
-        assert not t.is_unboxed, "{} unexpected unboxed type".format(t)
+        assert not t.is_unboxed, f"{t} unexpected unboxed type"
         return 'O'
 
     def visit_rtuple(self, t: 'RTuple') -> str:
@@ -468,8 +488,8 @@ class RTuple(RType):
         # in the same way python can just assign a Tuple[int, bool] to a Tuple[int, bool].
         self.unique_id = self.accept(TupleNameVisitor())
         # Nominally the max c length is 31 chars, but I'm not honestly worried about this.
-        self.struct_name = 'tuple_{}'.format(self.unique_id)
-        self._ctype = '{}'.format(self.struct_name)
+        self.struct_name = f'tuple_{self.unique_id}'
+        self._ctype = f'{self.struct_name}'
 
     def accept(self, visitor: 'RTypeVisitor[T]') -> T:
         return visitor.visit_rtuple(self)
@@ -502,11 +522,11 @@ exc_rtuple = RTuple([object_rprimitive, object_rprimitive, object_rprimitive])
 # Dictionary iterator tuple: (should continue, internal offset, key, value)
 # See mypyc.irbuild.for_helpers.ForDictionaryCommon for more details.
 dict_next_rtuple_pair = RTuple(
-    [bool_rprimitive, int_rprimitive, object_rprimitive, object_rprimitive]
+    [bool_rprimitive, short_int_rprimitive, object_rprimitive, object_rprimitive]
 )
 # Same as above but just for key or value.
 dict_next_rtuple_single = RTuple(
-    [bool_rprimitive, int_rprimitive, object_rprimitive]
+    [bool_rprimitive, short_int_rprimitive, object_rprimitive]
 )
 
 
@@ -528,7 +548,7 @@ def compute_rtype_alignment(typ: RType) -> int:
             items = typ.types
         else:
             assert False, "invalid rtype for computing alignment"
-        max_alignment = max([compute_rtype_alignment(item) for item in items])
+        max_alignment = max(compute_rtype_alignment(item) for item in items)
         return max_alignment
 
 
@@ -601,13 +621,15 @@ class RStruct(RType):
         return visitor.visit_rstruct(self)
 
     def __str__(self) -> str:
-        # if not tuple(unamed structs)
-        return '%s{%s}' % (self.name, ', '.join(name + ":" + str(typ)
+        # if not tuple(unnamed structs)
+        return '{}{{{}}}'.format(self.name, ', '.join(name + ":" + str(typ)
                                                 for name, typ in zip(self.names, self.types)))
 
     def __repr__(self) -> str:
-        return '<RStruct %s{%s}>' % (self.name, ', '.join(name + ":" + repr(typ) for name, typ
-                                                          in zip(self.names, self.types)))
+        return '<RStruct {}{{{}}}>'.format(
+            self.name, ', '.join(name + ":" + repr(typ)
+            for name, typ in zip(self.names, self.types))
+        )
 
     def __eq__(self, other: object) -> bool:
         return (isinstance(other, RStruct) and self.name == other.name
@@ -754,10 +776,10 @@ class RArray(RType):
         return visitor.visit_rarray(self)
 
     def __str__(self) -> str:
-        return '%s[%s]' % (self.item_type, self.length)
+        return f'{self.item_type}[{self.length}]'
 
     def __repr__(self) -> str:
-        return '<RArray %r[%s]>' % (self.item_type, self.length)
+        return f'<RArray {self.item_type!r}[{self.length}]>'
 
     def __eq__(self, other: object) -> bool:
         return (isinstance(other, RArray) and self.item_type == other.item_type
@@ -805,5 +827,5 @@ PySetObject = RStruct(
 PyListObject = RStruct(
     name='PyListObject',
     names=['ob_base', 'ob_item', 'allocated'],
-    types=[PyObject, pointer_rprimitive, c_pyssize_t_rprimitive]
+    types=[PyVarObject, pointer_rprimitive, c_pyssize_t_rprimitive]
 )
